@@ -47,18 +47,18 @@ class FacebookUser(auth_models.User):
     def access_token_expiration_date(self):
         return self._get_token_object().expiration_date
 
-    def _get_token_object(self):
-        return UserTokenManager.get_access_token(self.user_id)
-
     @property
     def graph(self):
         return graph_api.ObservableGraphAPI(self._get_token_object().token)
 
+    def _get_token_object(self):
+        return UserTokenManager.get_access_token(self.user_id)
+
     @property
     def js_session(self):
         return json.dumps({
             'access_token': self.access_token,
-            'uid': self.user_id
+            'uid': self.user_id,
         })
 
     @property
@@ -66,8 +66,9 @@ class FacebookUser(auth_models.User):
         response = utils.get_from_graph_api(self.graph, "me/friends")
         if 'data' in response:
             return response['data']
-        logger.warning("OpenGraph error: %s" % response)
-        return []
+        else:
+            logger.warning("OpenGraph error: %s" % response)
+            return []
 
     def update_app_friends(self):
         friends = self.friends
@@ -100,7 +101,7 @@ class UserTokenManager(object):
                     'expiration_date': expiration_date}
         obj, created = UserToken.objects.get_or_create(
             token=token, defaults=defaults)
-        if not created and expiration_date:
+        if not created:
             if obj.expiration_date > expiration_date + timedelta(seconds=30):
                 extra = {'object_expiration_date': obj.expiration_date,
                          'expiration_date': expiration_date,
@@ -227,31 +228,34 @@ def dispatch_engines_run(sender, instance, created, **kwargs):
 def debug_all_tokens_for_user(user_id):
     manager = FacebookTokenManager()
     token_manager = UserTokenManager()
-    user_tokens = UserToken.objects.filter(provider_user_id=user_id,
-        deleted=False)
-    processed_user_tokens = []
+    user_tokens = list(
+        UserToken.objects
+        .filter(provider_user_id=user_id, deleted=False)
+        .values_list('token', flat=True)
+    )
     for token in user_tokens:
-        processed_user_tokens.append(token.id)
         try:
-            data = manager.debug_token(token.token)
+            data = manager.debug_token(token)
         except ValueError:
             logger.info('Invalid access token')
-            token_manager.invalidate_access_token(token.token)
+            token_manager.invalidate_access_token(token)
         else:
             token_manager.insert_token(user_id, data.token, data.expires)
-
     try:
         best_token = token_manager.get_access_token(user_id)
     except UserToken.DoesNotExist:
-        pass
+        logger.info("Best token was deleted by other process.")
     else:
-        if best_token.id not in processed_user_tokens:
-            logger.info('Retrying debug_all_tokens_for_user.')
+        if best_token.token not in user_tokens:
+            logger.info(
+                'New best token has arrived.'
+                'Retrying debug_all_tokens_for_user.'
+            )
             debug_all_tokens_for_user.retry(args=[user_id],
                                             countdown=45)
         else:
             logger.info('Deleting user tokens except best one.')
-            tokens_to_delete = sorted(processed_user_tokens)
-            tokens_to_delete.remove(best_token.id)
-            for token_id in processed_user_tokens:
-                UserToken.objects.filter(id=token_id).update(deleted=True)
+            tokens_to_delete = sorted(user_tokens)
+            tokens_to_delete.remove(best_token.token)
+            for token in tokens_to_delete:
+                token_manager.invalidate_access_token(token)
