@@ -1,4 +1,3 @@
-import codecs
 import json
 try:
     from urllib.parse import urljoin
@@ -12,23 +11,38 @@ from django.conf.urls import patterns
 from django.conf.urls import url
 from django.core import signing
 from django.core.urlresolvers import reverse
+from django.utils import encoding
+
 
 class InvalidNextUrl(Exception):
     pass
 
-class Next():
+
+class Next(object):
     salt = 'facebook_auth.urls.Next'
 
     def encode(self, data):
-        data = signing.dumps(data, salt=self.salt)
+        data = self.dumps(data)
         return urlencode({'next': data})
 
     def decode(self, data):
         try:
-            return signing.loads(data, salt=self.salt)
+            return self.loads(data)
         except signing.BadSignature:
             raise InvalidNextUrl()
 
+    def dumps(self, obj):
+        data = json.dumps(
+            obj, separators=(',', ':'), sort_keys=True).encode('utf-8')
+        base64d = signing.b64_encode(data)
+        return signing.Signer(salt=self.salt).sign(base64d)
+
+    def loads(self, s):
+        base64d = encoding.force_bytes(
+            signing.Signer(salt=self.salt).unsign(s))
+        data = signing.b64_decode(base64d)
+        return json.loads(data.decode('utf-8'))
+
 
 def redirect_uri(next, close):
     return urljoin(
